"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jobIndex_1 = require("./jobIndex");
const normaliseJobs = (rawJobList) => {
    // convert raw jobs (which may just be strings) into Jobs (which are always objects), returning a new list
    // example jobs = [
    //         {"build":{"filters":{"branches":{"ignore":["master"]}}}},
    //         "lint",
    //         {"test":{"name": "test_dev", requires":["lint"]}}
    //        ]
    const normalisedJobsList = [];
    // for all jobs
    for (const jobCount in rawJobList) {
        let job = rawJobList[jobCount];
        if (typeof job === "string") {
            const outputJob = { [job]: { name: job } };
            normalisedJobsList.push(outputJob);
        }
        else {
            normalisedJobsList.push(job);
        }
    }
    return normalisedJobsList;
};
const getJobNameAndAttributes = (job) => {
    const jobName = Object.keys(job)[0]; // there should be only one
    const jobAttrs = job[jobName];
    return [jobName, jobAttrs];
};
const indexAllJobs = (jobIndex, jobs) => {
    // for all jobs
    for (const jobCount in jobs) {
        let job = jobs[jobCount];
        let [jobName, jobAttrs] = getJobNameAndAttributes(job);
        if ("name" in jobAttrs) {
            jobName = jobAttrs.name;
            job = { [jobName]: jobAttrs };
            jobs[jobCount] = job;
        }
        jobIndex.addJob(jobName, job);
    }
};
const normaliseAllRequiresLists = (jobIndex, jobs) => {
    // Replaces all strings in all requires lists with jobRefs.
    // If a requires values does not refer to exactly ONE job, raise an error.
    // for all jobs
    for (const jobCount in jobs) {
        let job = jobs[jobCount];
        let [jobName, jobAttrs] = getJobNameAndAttributes(job);
        if (!("requires" in jobAttrs))
            continue;
        const requirementsList = jobAttrs.requires;
        for (const requirementCount in requirementsList) {
            let requiredJobName = requirementsList[requirementCount];
            const jobList = jobIndex.getJobsWithName(requiredJobName);
            if (jobList.length !== 1) {
                throw new Error(`Error: Job '${jobName}' requires '${requiredJobName}', which is the name of ${jobList.length} other jobs in workflow`);
            }
            requirementsList[requirementCount] = jobList[0]; // there is only one
        }
    }
};
const renameJob = (job, newName) => {
    let [jobName, jobAttrs] = getJobNameAndAttributes(job);
    delete job[jobName];
    job[newName] = jobAttrs;
};
const renameDuplicates = (jobIndex) => {
    let jobsWereRenamed;
    do {
        jobsWereRenamed = false;
        const jobNameList = jobIndex.getJobNames();
        for (const jobNameCount in jobNameList) {
            const jobName = jobNameList[jobNameCount];
            const jobList = jobIndex.getJobsWithName(jobName);
            if (jobList.length > 1) {
                // there are duplicates, add increasing suffix to each job name
                for (const jobCount in jobList) {
                    const job = jobList[jobCount];
                    const suffix = Number(jobCount) + 1;
                    const newJobName = `${jobName}-${suffix}`;
                    renameJob(job, newJobName);
                    jobIndex.addJob(newJobName, job);
                }
                jobIndex.deleteJob(jobName);
                jobsWereRenamed = true;
            }
        }
    } while (jobsWereRenamed);
};
const getJobs = (rawJobList) => {
    const jobIndex = new jobIndex_1.JobIndex();
    const jobs = normaliseJobs(rawJobList);
    indexAllJobs(jobIndex, jobs);
    normaliseAllRequiresLists(jobIndex, jobs);
    renameDuplicates(jobIndex);
    return jobs;
};
exports.default = { getJobs };
//# sourceMappingURL=workflow.js.map